use crate::MainDialogue;
use futures::future::BoxFuture;
use sea_orm::prelude::*;
use std::{future::Future, sync::Arc};
use teloxide::{adaptors::Throttle, prelude::*};
use tokio::sync::Mutex;

type DynHanlder<T> = Box<
    dyn FnOnce(
            Throttle<Bot>,
            Message,
            DatabaseConnection,
            MainDialogue,
            T,
        ) -> BoxFuture<'static, crate::Result<()>>
        + Send,
>;

pub struct Handler<T> {
    pub func: Option<DynHanlder<T>>,

    pub previous: Option<Message>,
}

pub type PackagedHandler<T> = Arc<Mutex<Handler<T>>>;

impl<T> Handler<T> {
    /// Convinience method to convert a simple async function and a previous message into PackagedHandler
    #[inline]
    pub fn new<H, F>(f: H, previous: impl Into<Option<Message>>) -> PackagedHandler<T>
    where
        H: FnOnce(Throttle<Bot>, Message, DatabaseConnection, MainDialogue, T) -> F
            + Send
            + 'static,
        F: Future<Output = crate::Result<()>> + Send + 'static,
    {
        let handler = Self {
            func: Some(Box::new(|bot, msg, db, dialogue, val| {
                Box::pin(f(bot, msg, db, dialogue, val))
            })),
            previous: previous.into(),
        };
        Arc::new(Mutex::new(handler))
    }
}
